const join = require('path').join
const fs = require('fs')

/**
 * Returns 2D array of content data, each with its respective
 * registered directory, options, and nested files.
 */
module.exports = function getContentData (config) {
  const srcPath = config.contentPath
  const contentData = []
  config.dirs.forEach(type => {
    const dir = join('/', type[0])
    const opts = getDirOpts(type[1] || {}, config)
    const path = join(srcPath, dir)
    const otherDirs = getOtherRegisteredDirs(dir, config.dirs)

    const filesData = []
    fs.readdirSync(path).forEach(stat => {
      const statPath = join(path, stat)
      if(fs.statSync(statPath).isDirectory()) { // Nested Files
        const dirSection = join(dir, stat)
        if (!(otherDirs.indexOf(dirSection) > -1)) {
          filesData.push(...getFilesData(dirSection, srcPath, otherDirs))
        }
      }
      else { // Top Level file
        filesData.push({
          src: statPath,
          section: dir
        })
      }
    })
    contentData.push([dir, opts, filesData])
  })
  return contentData
}


/**
 * Recursively get all content files with their respective metadata.
 */
function getFilesData (contentDir, srcPath, blacklist, filesData = []) {
  const contentPath = join(srcPath, contentDir)

  fs.readdirSync(contentPath).forEach((stat, opts) => {
    const statPath = join(contentPath, stat)
    if(fs.statSync(statPath).isDirectory()) {
      const nestedContentDir = join(contentDir, stat)
      if (!(blacklist.indexOf(nestedContentDir) > -1)) {
        getFilesData(nestedContentDir, srcPath, blacklist, filesData)
      }
    } else {
      filesData.push({
        src: statPath,      // path to file
        section: contentDir // path to file directory
      })
    }
  })
  return filesData
}


/**
 * Gets content options via 1) directory options 2) global content config.
 */
function getDirOpts (dir, config) {
  const g = config.g
  const isPost = dir.isPost !== undefined ? dir.isPost : g.isPost
  return {
    route: join('/', dir.route || g.route),
    permalink: dir.permalink || g.permalink,
    isPost: !(isPost === false)
  }
}


/**
 * Gets all the registered content directory types.
 */
function  getOtherRegisteredDirs (currDir, contentTypes) {
  const dirs = []
  contentTypes.forEach(type => {
    const dir = type[0]
    if (dir !== currDir) dirs.push(join('/' + dir))
  })
  return dirs
}
